﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;

namespace BmsSabunSuggester
{
    /// <summary>
    /// ディレクトリの中にあるWavファイルとBmpファイルのリスト。
    /// </summary>
    class DirInfo
    {
        DirInfo()
        {
        }

        public Dictionary<string, string> WavFiles = new Dictionary<string, string>();
        public Dictionary<string, string> BmpFiles = new Dictionary<string, string>();
        public string FullPath;

        public bool ContainsWav(string path) { return WavFiles.ContainsKey(path); }
        public bool ContainsBmp(string path) { return BmpFiles.ContainsKey(path); }

        /// <summary>
        /// ディレクトリから
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static DirInfo ReadDirectory(string path)
        {
            DirInfo info = new DirInfo();

            info.FullPath = Path.GetFullPath(path);

            foreach (string p in Directory.GetFiles(path, "*.ogg"))
                info.WavFiles[Path.GetFileNameWithoutExtension(p)] = null;

            foreach (string p in Directory.GetFiles(path, "*.png"))
                info.BmpFiles[Path.GetFileNameWithoutExtension(p)] = null;

            return info;
        }

        /// <summary>
        /// シリアル値から
        /// </summary>
        /// <param name="serialized"></param>
        /// <returns></returns>
        public static DirInfo Load(string serialized)
        {
            DirInfo info = new DirInfo();

            string[] s = serialized.Split('*');
            info.FullPath = s[0];
            foreach (string f in s[1].Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries)) info.WavFiles.Add(f, null);
            foreach (string f in s[2].Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries)) info.BmpFiles.Add(f, null);

            return info;
        }

        /// <summary>
        /// キャッシュ
        /// </summary>
        string serializedCache;

        /// <summary>
        /// シリアルします。
        /// </summary>
        /// <returns></returns>
        public string Serialize()
        {
            if (serializedCache != null) return serializedCache;

            StringBuilder sb = new StringBuilder(4096);
            sb.Append(FullPath); sb.Append('*');
            foreach (string s in WavFiles.Keys) { sb.Append(s); sb.Append('/'); } sb.Append('*');
            foreach (string s in BmpFiles.Keys) { sb.Append(s); sb.Append('/'); } sb.Append('*');
            serializedCache = sb.ToString();
            return serializedCache;
        }

        public override string ToString()
        {
            return FullPath;
        }
    }
}
