﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using Tsukikage.BMSPlayer.Sequence;

namespace BmsSabunSuggester
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        string dropedBmsFilename;

        private void listBox1_DragDrop(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
            foreach (string fn in ((string[])e.Data.GetData(System.Windows.Forms.DataFormats.FileDrop, true)))
            {
                if (System.IO.Directory.Exists(fn))
                {
                    int count = dir.Count;
                    Invoke((Action<string>)AddDirectory, fn);
                    labelStatus.Text = "追加完了 " + (dir.Count - count) + "個のフォルダを読みました。\nBMSファイルをドロップするかフォルダを追加できます。";
                    dropedBmsFilename = null;
                }
                else
                {
                    Invoke((Action<string>)SearchBms, fn);
                    dropedBmsFilename = fn;
                }
            }

            return;
        }

        static List<DirInfo> dir = new List<DirInfo>();
        
        /// <summary>
        /// インストール先のディレクトリを追加します
        /// </summary>
        /// <param name="path"></param>
        void AddDirectory(string path)
        {
            path = Path.GetFullPath(path);
            labelStatus.Text = "追加中... " + path;
            Application.DoEvents();
            dir.Add(DirInfo.ReadDirectory(path));
            foreach (string d in Directory.GetDirectories(path))
                AddDirectory(d);
        }

        void SearchBms(string path)
        {
            labelStatus.Text = "BMS読み込み中...";

            Bms bms;
            using(FileStream fs = File.OpenRead(path))
                bms = new Bms(fs);
            
            bms.ParseInfo();
            List<string> wavData = new List<string>(1236);
            List<string> bmpData = new List<string>(1236);
            foreach (string s in bms.wavlist) if (s != null) wavData.Add(Path.GetFileNameWithoutExtension(s));
            foreach (string s in bms.bmplist) if (s != null) bmpData.Add(Path.GetFileNameWithoutExtension(s));

            if (wavData.Count == 0)
            {
                labelStatus.Text = "これわBMSですかね？　なんか、たぶん形式が違ってそうです...";
                return;
            }

            labelDropedInfo.Text = "インストール先を検索中: \n" + bms.infoData.ToString();

            List<DirInfo> cand = dir, candNext = new List<DirInfo>();
            for (int count = 1; count < 50; count++)
            {
                labelStatus.Text = "検索中... L" + count + " : " + cand.Count + "/" + dir.Count;
                candNext = cand.FindAll(d => d.ContainsWav(wavData[new Random().Next(wavData.Count)]));
                if (candNext.Count == 0) break;
                
                cand = candNext;
                if (cand.Count <= 1) break;
            }

            cand.RemoveAll(di => !Directory.Exists(di.FullPath));

            if (cand.Count == 0)
            {
                listBox1.Items.Clear();
                labelStatus.Text = "それっぽいフォルダが見つかりませんでした。";
            }
            else
            {
                labelStatus.Text = "集計中...";
                listBox1.Items.Clear();
                listBox1.Items.AddRange(cand.ToArray());
                labelStatus.Text = "完了 - この辺にインストールすれば良いと思います。";
            }
        }

        private void listBox1_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }


        private void buttonSave_Click(object sender, EventArgs e)
        {
            saveFileDialog1.DefaultExt = ".txt";
            saveFileDialog1.FileName = "DirDB.txt";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                using (StreamWriter sw = new StreamWriter(saveFileDialog1.FileName))
                {
                    sw.WriteLine("これはBMSフォルダのデータベースなの");
                    foreach (DirInfo i in dir)
                        sw.WriteLine(i.Serialize());
                }
            }
            labelStatus.Text = "OK." + dir.Count + "エントリを保存しました";
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            openFileDialog1.DefaultExt = ".txt";
            openFileDialog1.FileName = "DirDB.txt";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                dir = new List<DirInfo>();
                using (StreamReader sr = new StreamReader(openFileDialog1.FileName))
                {
                    if (sr.ReadLine() != "これはBMSフォルダのデータベースなの")
                    {
                        labelStatus.Text = "形式が違ってそうです...";
                        return;
                    }
                    while (!sr.EndOfStream)
                        dir.Add(DirInfo.Load(sr.ReadLine()));
                }
            }

            labelStatus.Text = "OK." + dir.Count + "エントリを読み込みました。\nインストールしたいBMSファイルをドロップするか、さらにフォルダを追加できます。";
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            DirInfo d = (DirInfo)listBox1.SelectedItem;
            if (d == null) return;

            if (!Directory.Exists(d.FullPath))
            {
                labelStatus.Text = "フォルダが見つかりません。消しました？";
                return;
            }
            
            labelStatus.Text = "フォルダ内のBMSを検索中...";
            Application.DoEvents();

            string dirBmsinfo = "選ばれたフォルダに含まれるBMSファイル:\n";
            List<Bms> bmss = new List<Bms>();

            foreach (string bmspath in Directory.GetFiles(d.FullPath, "*.bms"))
            {
                try
                {
                    using (FileStream fs = File.OpenRead(bmspath))
                    {
                        Bms b = new Bms(fs);
                        b.ParseInfo();
                        bmss.Add(b);
                    }
                }
                catch { }
            }

            foreach (string bmspath in Directory.GetFiles(d.FullPath, "*.bme"))
            {
                try
                {
                    using (FileStream fs = File.OpenRead(bmspath))
                    {
                        Bms b = new Bms(fs);
                        b.ParseInfo();
                        bmss.Add(b);
                    }
                }
                catch { }
            }

            bmss.Sort((a, b) => a.infoData.playlevel - b.infoData.playlevel);
            foreach (Bms bms in bmss)
                dirBmsinfo += bms.infoData.ToString() + "\n";

            labelStatus.Text = "ここにインストールするには、そこのボタンを押します。";
            labelSelectedInfo.Text = dirBmsinfo;
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            if (dropedBmsFilename == null) return;
            DirInfo d = (DirInfo)listBox1.SelectedItem;
            if (d == null) return;
            try
            {
                File.Copy(dropedBmsFilename, d.FullPath + "\\" + Path.GetFileName(dropedBmsFilename), true);
                labelStatus.Text = "BMSファイルをフォルダにコピーしました。";
            }
            catch (Exception ex)
            {
                labelStatus.Text = "BMSファイルのコピーに失敗しました。\n" + ex.Message;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            labelStatus.Text = "BMSフォルダをドロップするか、リストを読み込んでください。";
        }
    }
}
